from typing import final

from .base_application import BaseAppCtx, BaseLambdaApp, Luna3Session
from .configs.configs.services import SettingsLambdaUnit


class AppCtx[TConfig: SettingsLambdaUnit](BaseAppCtx):
    """Configurator application context"""

    lambdaType: str = "standalone"
    luna3Session: Luna3Session | None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.luna3session = Luna3Session(settings=self.serviceConfig, lambdaType=self.lambdaType)

    @final
    async def initialize(self):
        """Description see :func:`~_LunaApplication.initialize`"""
        await super().initialize()

    @final
    async def shutdown(self):
        await super().shutdown()


class StandaloneLambdaApp(BaseLambdaApp):
    """Standalone lambda app"""

    from ..public.standalone.standalone_request import StandaloneLambdaRequest

    # service context
    ctxCls = AppCtx
    # request class
    requestCls = StandaloneLambdaRequest
    # service config
    configCls = SettingsLambdaUnit
