"""
Configs migration

Generation datetime: 2021-01-20 14:34:36.227939
"""

from typing import List

from configs_core.migration.db_migrate_helper import DatabaseMigrationHelper as mh, LimitationToModify

REVISION = "bc48d6fc"
DOWN_REVISION = "7bb857ca"


def upgradeAttributesDBSetting(setting: dict) -> dict:
    """Upgrade AttributesDB setting."""
    setting.pop("channel", None)
    return setting


def downgradeAttributesDBSetting(setting: dict) -> dict:
    """Downgrade AttributesDB setting."""
    setting["channel"] = "luna-sender"
    return setting


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="LUNA_ATTRIBUTES_DB",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "host": {"type": "string"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "password": {"type": "string"},
                "channel": {"type": "string"},
                "sentinel": {
                    "description": "Redis sentinels settings",
                    "type": "object",
                    "properties": {
                        "master_name": {"type": "string"},
                        "sentinels": {
                            "type": "array",
                            "items": {
                                "description": "Redis sentinel settings. Host+port",
                                "type": "string",
                                "pattern": "[a-zA-Z0-9.]+:\\d{1,5}",
                                "example": "127.0.0.1:26379",
                            },
                        },
                    },
                    "additionalProperties": False,
                    "required": ["master_name", "sentinels"],
                },
                "number": {
                    "type": "integer",
                    "enum": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14],
                },
            },
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "host": {"type": "string"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "password": {"type": "string"},
                "sentinel": {
                    "description": "Redis sentinels settings",
                    "type": "object",
                    "properties": {
                        "master_name": {"type": "string"},
                        "sentinels": {
                            "type": "array",
                            "items": {
                                "description": "Redis sentinel settings. Host+port",
                                "type": "string",
                                "pattern": "[a-zA-Z0-9.]+:\\d{1,5}",
                                "example": "127.0.0.1:26379",
                            },
                        },
                    },
                    "additionalProperties": False,
                    "required": ["master_name", "sentinels"],
                },
                "number": {
                    "type": "integer",
                    "enum": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14],
                },
            },
            "additionalProperties": False,
        },
        oldDefaultValue={
            "host": "127.0.0.1",
            "port": 6379,
            "password": "",
            "channel": "luna-sender",
            "sentinel": {"master_name": "luna_attributes", "sentinels": []},
            "number": 0,
        },
        newDefaultValue={
            "host": "127.0.0.1",
            "port": 6379,
            "password": "",
            "sentinel": {"master_name": "luna_attributes", "sentinels": []},
            "number": 0,
        },
        settingUpgradeFunction=upgradeAttributesDBSetting,
        settingDowngradeFunction=downgradeAttributesDBSetting,
        # DON'T REMOVE. Used in profiled migrations.
        oldServices=["luna-faces", "luna-matcher-proxy", "luna-python-matcher"],
        newServices=["luna-faces", "luna-matcher-proxy", "luna-python-matcher"],
    ),
]


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)
