"""
Configs migration

Generation datetime: 2022-03-23 16:32:18.965630
"""

from copy import deepcopy
from typing import List

from configs_core.migration.db_migrate_helper import DatabaseMigrationHelper, LimitationToModify

mh: DatabaseMigrationHelper

REVISION = "93722463"
DOWN_REVISION = "41164640"


def upgradeDescriptorsCache(settings: dict):
    """Adds new fields."""
    res = deepcopy(settings)
    res["matching_settings"]["batch_size"] = 20
    res["matching_settings"]["tasks_count"] = 10
    res["rpc_settings"]["pool_size"] = 100
    return res


def downgradeDescriptorsCache(settings: dict):
    """Remove fields."""
    res = deepcopy(settings)
    del res["matching_settings"]["batch_size"]
    del res["matching_settings"]["tasks_count"]
    del res["rpc_settings"]["pool_size"]
    return res


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="DESCRIPTORS_CACHE",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "cache_enabled": {"type": "boolean"},
                "updating_cache_interval": {"type": "number", "exclusiveMinimum": 0},
                "matching_settings": {
                    "type": "object",
                    "properties": {"thread_count": {"type": "integer", "minimum": 0}},
                    "description": "matcher lib settings",
                    "additionalProperties": False,
                },
                "rpc_settings": {
                    "type": "object",
                    "properties": {
                        "timeouts": {
                            "type": "object",
                            "properties": {
                                "connect_timeout": {"type": "integer", "minimum": 0},
                                "request_timeout": {"type": "integer", "minimum": 0},
                                "response_timeout": {"type": "integer", "minimum": 0},
                            },
                            "description": "matcher lib settings",
                            "additionalProperties": False,
                        }
                    },
                    "description": "matcher lib settings",
                    "additionalProperties": False,
                },
            },
            "description": "python matcher descriptor caching settings",
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "cache_enabled": {"type": "boolean"},
                "updating_cache_interval": {"type": "number", "exclusiveMinimum": 0},
                "matching_settings": {
                    "type": "object",
                    "properties": {
                        "thread_count": {"type": "integer", "minimum": 0},
                        "tasks_count": {"type": "integer", "minimum": 0},
                        "batch_size": {"type": "integer", "minimum": 0},
                    },
                    "description": "matcher lib settings",
                    "additionalProperties": False,
                },
                "rpc_settings": {
                    "type": "object",
                    "properties": {
                        "timeouts": {
                            "type": "object",
                            "properties": {
                                "connect_timeout": {"type": "integer", "minimum": 0},
                                "request_timeout": {"type": "integer", "minimum": 0},
                                "response_timeout": {"type": "integer", "minimum": 0},
                            },
                            "description": "matcher lib settings",
                            "additionalProperties": False,
                        },
                        "pool_size": {"type": "integer", "minimum": 0},
                    },
                    "description": "matcher lib settings",
                    "additionalProperties": False,
                },
            },
            "description": "python matcher descriptor caching settings",
            "additionalProperties": False,
        },
        oldDefaultValue={
            "cache_enabled": True,
            "updating_cache_interval": 2,
            "matching_settings": {"thread_count": 0},
            "rpc_settings": {
                "timeouts": {
                    "connect_timeout": 20,
                    "request_timeout": 60,
                    "response_timeout": 60,
                }
            },
        },
        newDefaultValue={
            "cache_enabled": True,
            "updating_cache_interval": 2,
            "matching_settings": {
                "thread_count": 0,
                "batch_size": 20,
                "tasks_count": 10,
            },
            "rpc_settings": {
                "timeouts": {
                    "connect_timeout": 20,
                    "request_timeout": 60,
                    "response_timeout": 60,
                },
                "pool_size": 100,
            },
        },
        settingUpgradeFunction=upgradeDescriptorsCache,
        settingDowngradeFunction=downgradeDescriptorsCache,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-python-matcher"],
        newServices=["luna-python-matcher"],
    ),
]


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)
