"""
Configs migration

Generation datetime: 2024-03-06 14:19:25.696613
"""

from typing import List

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "182ecb3a"
DOWN_REVISION = "080a1b4d"


def upgradeFunction(setting: dict):
    setting.get("estimator_settings", {}).pop("landmarks17_threshold", None)
    return setting


def downgradeFunction(setting: dict):
    estimatorSettings = setting.get("estimator_settings")
    if estimatorSettings:
        estimatorSettings["landmarks17_threshold"] = 0.25
    return setting


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_BODY_DETECTOR_SETTINGS",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "runtime_settings": {
                    "type": "object",
                    "properties": {
                        "device_class": {
                            "type": "string",
                            "enum": ["global", "cpu", "gpu"],
                        },
                        "optimal_batch_size": {"type": "integer", "minimum": 1},
                        "worker_count": {"type": "integer", "minimum": 1},
                    },
                    "additionalProperties": False,
                },
                "estimator_settings": {
                    "type": "object",
                    "properties": {
                        "image_size": {"type": "integer", "minimum": 1},
                        "redetect_score_threshold": {"type": "number"},
                        "score_threshold": {"type": "number"},
                        "landmarks17_threshold": {"type": "number"},
                    },
                    "description": "settings for human body detector",
                    "additionalProperties": False,
                },
            },
            "description": "settings for body detector",
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "runtime_settings": {
                    "type": "object",
                    "properties": {
                        "device_class": {
                            "type": "string",
                            "enum": ["global", "cpu", "gpu"],
                        },
                        "optimal_batch_size": {"type": "integer", "minimum": 1},
                        "worker_count": {"type": "integer", "minimum": 1},
                    },
                    "additionalProperties": False,
                },
                "estimator_settings": {
                    "type": "object",
                    "properties": {
                        "image_size": {"type": "integer", "minimum": 1},
                        "redetect_score_threshold": {"type": "number"},
                        "score_threshold": {"type": "number"},
                    },
                    "description": "settings for human body detector",
                    "additionalProperties": False,
                },
            },
            "description": "settings for body detector",
            "additionalProperties": False,
        },
        oldDefaultValue={
            "runtime_settings": {
                "device_class": "global",
                "optimal_batch_size": 10,
                "worker_count": 1,
            },
            "estimator_settings": {
                "image_size": 640,
                "redetect_score_threshold": 0.12,
                "score_threshold": 0.5,
                "landmarks17_threshold": 0.25,
            },
        },
        newDefaultValue={
            "runtime_settings": {
                "device_class": "global",
                "optimal_batch_size": 10,
                "worker_count": 1,
            },
            "estimator_settings": {
                "image_size": 640,
                "redetect_score_threshold": 0.12,
                "score_threshold": 0.5,
            },
        },
        settingUpgradeFunction=upgradeFunction,
        settingDowngradeFunction=downgradeFunction,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-remote-sdk"],
        newServices=["luna-remote-sdk"],
    ),
]

externalLimitationsToModify: List[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)
