"""
Configs migration

Generation datetime: 2024-06-13 09:42:49.179146
"""

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "391471e5"
DOWN_REVISION = "94742577"


limitationsToModify: list[LimitationToModify] = []

externalLimitationsToModify: list[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to add
    mh.addLimitation(
        limitationName="DESCRIPTOR_ENCRYPTION",
        validationSchema={
            "oneOf": [
                {
                    "type": "object",
                    "properties": {
                        "enabled": {"type": "boolean"},
                        "algorithm": {"type": "string", "enum": ["aes256-gcm"]},
                        "params": {
                            "type": "object",
                            "properties": {
                                "source": {"type": "string", "enum": ["vaultKV"]},
                                "key": {
                                    "type": "object",
                                    "properties": {
                                        "url": {"type": "string"},
                                        "token": {"type": "string"},
                                    },
                                    "additionalProperties": False,
                                },
                            },
                            "required": ["source", "key"],
                        },
                    },
                    "additionalProperties": False,
                    "default": {"enabled": False},
                },
                {
                    "type": "object",
                    "properties": {
                        "enabled": {"type": "boolean"},
                        "algorithm": {"type": "string", "enum": ["aes256-gcm"]},
                        "params": {
                            "type": "object",
                            "properties": {
                                "source": {"type": "string", "enum": ["raw"]},
                                "key": {"type": "string"},
                            },
                            "required": ["source", "key"],
                            "additionalProperties": False,
                        },
                    },
                    "additionalProperties": False,
                    "default": {"enabled": False},
                },
            ]
        },
        defaultValue={
            "enabled": False,
            "algorithm": "aes256-gcm",
            "params": {"source": "raw", "key": ""},
        },
        description="Descriptor encryption settings",
        services=[
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
        ],
        groupNames=["luna-platform"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to remove
    mh.removeLimitation(limitationName="DESCRIPTOR_ENCRYPTION")
