"""
Configs migration

Generation datetime: 2024-10-13 14:35:59.973189
"""

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "5202299a"
DOWN_REVISION = "4e314cea"


limitationsToModify: list[LimitationToModify] = [
    LimitationToModify(
        limitationName="TASKS_TO_MATCHER_CONCURRENCY",
        oldValidationSchema={"type": "integer", "minimum": 1},
        newValidationSchema={"type": "integer", "minimum": 1},
        oldDefaultValue=2,
        newDefaultValue=2,
        oldDescription="Tasks to matcher request concurrency.",
        newDescription="Tasks to matcher request concurrency.",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform"],
        settingUpgradeFunction=None,
        settingDowngradeFunction=None,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-tasks"],
        newServices=["luna-lambda-tasks-unit", "luna-tasks"],
    ),
]

externalLimitationsToModify: list[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)
