"""
Configs migration

Generation datetime: 2024-12-04 14:31:39.023972
"""

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "fdb9b605"
DOWN_REVISION = "e14dedfa"


def upgradeASUSetting(setting: dict) -> dict:
    """Upgrade ASU setting."""
    setting["luna_streams_retranslator"] = False
    return setting


def downgradeASU(setting: dict) -> dict:
    """Downgrade ASU setting."""
    setting.pop("luna_streams_retranslator", None)
    return setting


limitationsToModify: list[LimitationToModify] = [
    LimitationToModify(
        limitationName="LUNA_MONITORING",
        oldValidationSchema={
            "oneOf": [
                {
                    "properties": {
                        "storage_type": {"type": "string", "enum": ["influx"]},
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "organization": {"type": "string"},
                        "token": {"type": "string"},
                        "bucket": {"type": "string"},
                    },
                    "default": {
                        "storage_type": "influx",
                        "send_data_for_monitoring": 1,
                        "use_ssl": 0,
                        "flushing_period": 1,
                        "host": "127.0.0.1",
                        "port": 8086,
                        "organization": "luna",
                        "token": "secret-token",
                        "bucket": "luna_monitoring",
                    },
                    "required": [
                        "storage_type",
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "organization",
                        "token",
                        "bucket",
                    ],
                    "type": "object",
                    "additionalProperties": False,
                    "description": "Influx monitoring settings.",
                }
            ]
        },
        newValidationSchema={
            "oneOf": [
                {
                    "properties": {
                        "storage_type": {"type": "string", "enum": ["influx"]},
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "organization": {"type": "string"},
                        "token": {"type": "string"},
                        "bucket": {"type": "string"},
                    },
                    "default": {
                        "storage_type": "influx",
                        "send_data_for_monitoring": 1,
                        "use_ssl": 0,
                        "flushing_period": 1,
                        "host": "127.0.0.1",
                        "port": 8086,
                        "organization": "luna",
                        "token": "secret-token",
                        "bucket": "luna_monitoring",
                    },
                    "required": [
                        "storage_type",
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "organization",
                        "token",
                        "bucket",
                    ],
                    "type": "object",
                    "additionalProperties": False,
                    "description": "Influx monitoring settings.",
                }
            ]
        },
        oldDefaultValue={
            "storage_type": "influx",
            "send_data_for_monitoring": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "bucket": "luna_monitoring",
            "organization": "luna",
            "token": "secret-token",
        },
        newDefaultValue={
            "storage_type": "influx",
            "send_data_for_monitoring": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "bucket": "luna_monitoring",
            "organization": "luna",
            "token": "secret-token",
        },
        oldDescription="monitoring parameters",
        newDescription="monitoring parameters",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform"],
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-lambda",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-sender",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
        newServices=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-lambda",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-sender",
            "luna-streams-retranslator",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
    ),
    LimitationToModify(
        limitationName="LUNA_SERVICE_METRICS",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "enabled": {"type": "boolean"},
                "metrics_format": {"type": "string"},
                "extra_labels": {
                    "type": "object",
                    "properties": {},
                    "additionalProperties": True,
                },
            },
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "enabled": {"type": "boolean"},
                "metrics_format": {"type": "string"},
                "extra_labels": {
                    "type": "object",
                    "properties": {},
                    "additionalProperties": True,
                },
            },
            "additionalProperties": False,
        },
        oldDefaultValue={
            "enabled": False,
            "metrics_format": "prometheus",
            "extra_labels": {},
        },
        newDefaultValue={
            "enabled": False,
            "metrics_format": "prometheus",
            "extra_labels": {},
        },
        oldDescription="Metrics collection configuration",
        newDescription="Metrics collection configuration",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform"],
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-lambda",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-sender",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
        newServices=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-lambda",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-sender",
            "luna-streams-retranslator",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
    ),
    LimitationToModify(
        limitationName="ADDITIONAL_SERVICES_USAGE",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "luna_sender": {"type": "boolean"},
                "luna_events": {"type": "boolean"},
                "luna_tasks": {"type": "boolean"},
                "luna_matcher_proxy": {"type": "boolean"},
                "luna_image_store": {"type": "boolean"},
                "luna_handlers": {"type": "boolean"},
                "luna_lambda": {"type": "boolean"},
                "luna_video_manager": {"type": "boolean"},
                "luna_video_agent": {"type": "boolean"},
            },
            "required": [
                "luna_sender",
                "luna_events",
                "luna_tasks",
                "luna_matcher_proxy",
                "luna_image_store",
                "luna_handlers",
                "luna_lambda",
                "luna_video_manager",
                "luna_video_agent",
            ],
            "description": "whether to use nested optional services",
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "luna_sender": {"type": "boolean"},
                "luna_events": {"type": "boolean"},
                "luna_tasks": {"type": "boolean"},
                "luna_matcher_proxy": {"type": "boolean"},
                "luna_image_store": {"type": "boolean"},
                "luna_handlers": {"type": "boolean"},
                "luna_lambda": {"type": "boolean"},
                "luna_video_manager": {"type": "boolean"},
                "luna_video_agent": {"type": "boolean"},
                "luna_streams_retranslator": {"type": "boolean"},
            },
            "required": [
                "luna_sender",
                "luna_events",
                "luna_tasks",
                "luna_matcher_proxy",
                "luna_image_store",
                "luna_handlers",
                "luna_lambda",
                "luna_video_manager",
                "luna_video_agent",
                "luna_streams_retranslator",
            ],
            "description": "whether to use nested optional services",
            "additionalProperties": False,
        },
        oldDefaultValue={
            "luna_sender": True,
            "luna_events": True,
            "luna_tasks": True,
            "luna_matcher_proxy": False,
            "luna_image_store": True,
            "luna_handlers": True,
            "luna_lambda": False,
            "luna_video_manager": False,
            "luna_video_agent": False,
        },
        newDefaultValue={
            "luna_sender": True,
            "luna_events": True,
            "luna_tasks": True,
            "luna_matcher_proxy": False,
            "luna_image_store": True,
            "luna_handlers": True,
            "luna_lambda": False,
            "luna_video_manager": False,
            "luna_video_agent": False,
            "luna_streams_retranslator": False,
        },
        oldDescription="whether to use nested optional services",
        newDescription="whether to use nested optional services",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform"],
        settingUpgradeFunction=upgradeASUSetting,
        settingDowngradeFunction=downgradeASU,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-admin",
            "luna-api",
            "luna-handlers",
            "luna-lambda",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-tasks",
            "luna-video-agent",
        ],
        newServices=[
            "luna-admin",
            "luna-api",
            "luna-handlers",
            "luna-lambda",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
    ),
]

externalLimitationsToModify: list[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to add
    mh.addLimitation(
        limitationName="EXTERNAL_LUNA_STREAMS_HLS_RETRANSMISSION_ADDRESS",
        validationSchema={
            "type": "object",
            "properties": {
                "host": {"type": "string"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
            },
            "required": ["host", "port"],
            "additionalProperties": False,
        },
        defaultValue={"host": "http://127.0.0.1", "port": 8888},
        description="external luna streams hls retransmission address",
        services=["luna-video-manager"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_RETRANSLATOR_DB_ADDRESS",
        validationSchema={
            "type": "object",
            "properties": {
                "host": {"type": "string"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "user": {"type": "string"},
                "password": {"type": "string"},
                "sentinel": {
                    "description": "Redis sentinels settings",
                    "type": "object",
                    "properties": {
                        "master_name": {"type": "string"},
                        "sentinels": {
                            "type": "array",
                            "items": {
                                "description": "Redis sentinel settings. Host+port",
                                "type": "string",
                                "pattern": "[a-zA-Z0-9.]+:\\d{1,5}",
                                "example": "127.0.0.1:26379",
                            },
                        },
                        "user": {"type": "string"},
                        "password": {"type": "string"},
                    },
                    "additionalProperties": False,
                    "required": ["master_name", "sentinels", "user", "password"],
                },
                "number": {
                    "type": "integer",
                    "enum": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14],
                },
            },
            "additionalProperties": False,
        },
        defaultValue={
            "host": "127.0.0.1",
            "port": 6379,
            "user": "",
            "password": "",
            "sentinel": {
                "master_name": "luna_retranslator",
                "sentinels": [],
                "user": "",
                "password": "",
            },
            "number": 0,
        },
        description="luna retranslator database address",
        services=["luna-streams-retranslator"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_STREAMS_RETRANSLATOR_ACTIVE_PLUGINS",
        validationSchema={"type": "array", "items": {"type": "string"}},
        defaultValue=[],
        description="list of luna-streams-retranslator active plugins",
        services=["luna-streams-retranslator"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_STREAMS_RETRANSLATOR_ADDRESS",
        validationSchema={
            "type": "object",
            "properties": {
                "origin": {
                    "type": "string",
                    "pattern": "^http(s)?:\\/\\/[^\\/:]+(:\\d+)?$",
                    "example": "http://127.0.0.1:5051",
                },
                "api_version": {"type": "integer"},
            },
            "required": ["origin", "api_version"],
            "additionalProperties": False,
        },
        defaultValue={"origin": "http://127.0.0.1:5250", "api_version": 1},
        description="LUNA Streams Retranslator address",
        services=["luna-admin", "luna-api", "luna-video-manager"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_STREAMS_RETRANSLATOR_HTTP_SETTINGS",
        validationSchema={
            "type": "object",
            "properties": {
                "request_timeout": {"type": "integer", "minimum": 1},
                "response_timeout": {"type": "integer", "minimum": 1},
                "request_max_size": {"type": "integer", "minimum": 1},
                "keep_alive_timeout": {"type": "integer", "minimum": 1},
            },
            "additionalProperties": False,
        },
        defaultValue={
            "request_timeout": 60,
            "response_timeout": 600,
            "request_max_size": 1073741824,
            "keep_alive_timeout": 200,
        },
        description="Luna Streams Retranslator http settings",
        services=["luna-streams-retranslator"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_STREAMS_RETRANSLATOR_IGNORED_RESTREAM_TTL",
        validationSchema={"type": "integer"},
        defaultValue=60,
        description="ttl of not used restream",
        services=["luna-streams-retranslator"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_STREAMS_RETRANSLATOR_LOGGER",
        validationSchema={
            "type": "object",
            "properties": {
                "log_level": {
                    "type": "string",
                    "enum": ["ERROR", "WARNING", "INFO", "DEBUG"],
                },
                "folder_with_logs": {"type": "string"},
                "max_log_file_size": {"type": "integer"},
                "log_time": {"type": "string", "enum": ["LOCAL", "UTC"]},
                "log_to_stdout": {"type": "boolean"},
                "log_to_file": {"type": "boolean"},
                "multiline_stack_trace": {"type": "boolean"},
                "format": {"type": "string", "enum": ["default", "json", "ecs"]},
            },
            "additionalProperties": False,
        },
        defaultValue={
            "log_level": "INFO",
            "folder_with_logs": "./",
            "max_log_file_size": 1024,
            "log_time": "LOCAL",
            "log_to_stdout": True,
            "log_to_file": False,
            "multiline_stack_trace": True,
            "format": "default",
        },
        description="Luna Streams Retranslator logger settings",
        services=["luna-streams-retranslator"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_STREAMS_RETRANSLATOR_TIMEOUTS",
        validationSchema={
            "type": "object",
            "properties": {
                "request": {"type": "number", "exclusiveMinimum": 0},
                "connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_read": {"type": "number", "exclusiveMinimum": 0},
            },
            "description": "timeouts in seconds",
            "additionalProperties": False,
        },
        defaultValue={
            "request": 60,
            "connect": 20,
            "sock_connect": 10,
            "sock_read": 60,
        },
        description="timeouts for http-request to luna-streams-retranslator service",
        services=["luna-api", "luna-video-manager"],
        groupNames=["luna-platform"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to remove
    mh.removeLimitation(limitationName="EXTERNAL_LUNA_STREAMS_HLS_RETRANSMISSION_ADDRESS")
    mh.removeLimitation(limitationName="LUNA_RETRANSLATOR_DB_ADDRESS")
    mh.removeLimitation(limitationName="LUNA_STREAMS_RETRANSLATOR_ACTIVE_PLUGINS")
    mh.removeLimitation(limitationName="LUNA_STREAMS_RETRANSLATOR_ADDRESS")
    mh.removeLimitation(limitationName="LUNA_STREAMS_RETRANSLATOR_HTTP_SETTINGS")
    mh.removeLimitation(limitationName="LUNA_STREAMS_RETRANSLATOR_IGNORED_RESTREAM_TTL")
    mh.removeLimitation(limitationName="LUNA_STREAMS_RETRANSLATOR_LOGGER")
    mh.removeLimitation(limitationName="LUNA_STREAMS_RETRANSLATOR_TIMEOUTS")
