"""
Module contains settings class for luna-video-manager
"""

from typing import Optional

from ..settings.classes import (
    AdditionalServicesUsageSetting as _AdditionalServicesUsageSetting,
    AddressSetting as _AddressSetting,
    DBSetting as _DBSetting,
    ServiceAddressSettings as _ServiceAddressSettings,
    ServiceTimeoutsSettings as _ServiceTimeoutsSettings,
)
from .base_python_service_settings import BasePythonServiceSettingsClass


class SettingsVideoManager(BasePythonServiceSettingsClass):
    """
    Settings class for luna-video-manager
    """

    serviceName = "luna-video-manager"

    def __init__(self):
        super().__init__()
        self.db: Optional[_DBSetting] = None
        self.storageTime: Optional[str] = None
        self.streamStatusObsoletingPeriod: Optional[float] = None
        self.streamStatusObsoletingInterval: Optional[float] = None
        self.agentStatusObsoletingPeriod: Optional[float] = None
        self.agentStatusObsoletingInterval: Optional[float] = None
        self.streamsAgentSearchInterval: Optional[float] = None
        self.autoRestarterInterval: Optional[float] = None
        self.licensesAddress: Optional[_ServiceAddressSettings] = None
        self.additionalServicesUsage: Optional[_AdditionalServicesUsageSetting] = None
        self.streamsRetranslatorAddress: Optional[_ServiceAddressSettings] = None
        self.streamsRetranslatorTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.externalLunaStreamsHLSRetransmissionAddress: Optional[_AddressSetting] = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()
        self.db = _DBSetting(self.serviceName, self["LUNA_VIDEO_MANAGER_DB"])
        self.storageTime = self["storage_time"]
        self.streamStatusObsoletingPeriod = self["LUNA_VIDEO_MANAGER_STREAM_STATUS_OBSOLETING_PERIOD"]
        self.streamStatusObsoletingInterval = self["LUNA_VIDEO_MANAGER_STREAM_STATUS_OBSOLETING_INTERVAL"]
        self.agentStatusObsoletingPeriod = self["LUNA_VIDEO_MANAGER_AGENT_STATUS_OBSOLETING_PERIOD"]
        self.agentStatusObsoletingInterval = self["LUNA_VIDEO_MANAGER_AGENT_STATUS_OBSOLETING_INTERVAL"]
        self.streamsAgentSearchInterval = self["LUNA_VIDEO_MANAGER_STREAMS_AGENT_SEARCH_INTERVAL"]
        self.autoRestarterInterval = self["LUNA_VIDEO_MANAGER_STREAMS_AUTORESTARTER_INTERVAL"]
        self.licensesAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_LICENSES_ADDRESS", self["LUNA_LICENSES_ADDRESS"]
        )
        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName, self["ADDITIONAL_SERVICES_USAGE"]
        )
        self.streamsRetranslatorAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_STREAMS_RETRANSLATOR_ADDRESS", self["LUNA_STREAMS_RETRANSLATOR_ADDRESS"]
        )
        self.streamsRetranslatorTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_STREAMS_RETRANSLATOR_TIMEOUTS", self["LUNA_STREAMS_RETRANSLATOR_TIMEOUTS"]
        )
        self.externalLunaStreamsHLSRetransmissionAddress = _AddressSetting(
            self.serviceName,
            "EXTERNAL_LUNA_STREAMS_HLS_RETRANSMISSION_ADDRESS",
            self["EXTERNAL_LUNA_STREAMS_HLS_RETRANSMISSION_ADDRESS"],
        )
